#!/bin/sh -xe

OS=`facter operatingsystem`

# Activate the virtual environment
cd "$WORKSPACE"
source .venv/bin/activate

# Run pylint
echo "**> running pylint tests ..."
#TODO pylint -E

# Run unit tests
echo "**> running unit tests tests ..."
cd autotest

# Make sure the PostGIS test database is not present
# Nothing to do for SpatiaLite which uses an in-memory database
if [ $DB == "postgis" ] && [ `psql template_postgis jenkins -tAc "SELECT 1 FROM pg_database WHERE datname='test_eoxserver_testing'"` ] ; then
    echo "Dropping PostGIS test database."
    dropdb test_eoxserver_testing
fi

export XML_CATALOG_FILES="$WORKSPACE/schemas/catalog.xml"
# the WPS unit-tests cannot be run via manage.py test in Django <=1.5
python -m eoxserver.services.ows.wps.test_data_types -v
python -m eoxserver.services.ows.wps.test_allowed_values -v
python manage.py test autotest_services -v2
python manage.py test autotest_coverages -v2
python manage.py test --pythonpath=../eoxserver/ core -v2
#TODO python manage.py test --pythonpath=../eoxserver/ backends -v2
python manage.py test --pythonpath=../eoxserver/ services -v2
python manage.py test --pythonpath=../eoxserver/resources coverages -v2
#TODO python manage.py test --pythonpath=../eoxserver/resources processes -v2
cd ..

# Run command line tests
echo "**> running command line tests ..."
cd autotest_jenkins

# Reset PostGIS database if used
if [ $DB == "postgis" ]; then
    if [ `psql template_postgis jenkins -tAc "SELECT 1 FROM pg_database WHERE datname='eoxserver_testing'"` ] ; then
        echo "Dropping PostGIS database."
        dropdb eoxserver_testing
    fi
    createdb -T template_postgis -O jenkins eoxserver_testing
fi

python manage.py syncdb --noinput --traceback
python manage.py loaddata auth_data.json range_types.json --traceback
python manage.py eoxs_rangetype_load -i autotest_jenkins/data/meris/meris_range_type_definition.json --traceback # load a new range-type with forbiden update 
python manage.py eoxs_rangetype_load -u -i autotest_jenkins/data/asar/asar_range_type_definition.json --traceback # load a new range-type with allowed update
python manage.py eoxs_rangetype_load -u -i autotest_jenkins/data/meris/meris_range_type_definition.json --traceback # update an existing range-type
python manage.py eoxs_rangetype_list --traceback
python manage.py eoxs_rangetype_list --json --traceback # dump all range-types as JSON
python manage.py eoxs_collection_create -i MER_FRS_1P_reduced --traceback
python manage.py eoxs_dataset_register -d autotest_jenkins/data/meris/MER_FRS_1P_reduced/ENVISAT-MER_FRS_1PNPDE20060816_090929_000001972050_00222_23322_0058_uint16_reduced_compressed.tif -m autotest_jenkins/data/meris/MER_FRS_1P_reduced/ENVISAT-MER_FRS_1PNPDE20060816_090929_000001972050_00222_23322_0058_uint16_reduced_compressed.xml -r MERIS_uint16 --visible --traceback
python manage.py eoxs_dataset_register -d autotest_jenkins/data/meris/MER_FRS_1P_reduced/ENVISAT-MER_FRS_1PNPDE20060822_092058_000001972050_00308_23408_0077_uint16_reduced_compressed.tif -m autotest_jenkins/data/meris/MER_FRS_1P_reduced/ENVISAT-MER_FRS_1PNPDE20060822_092058_000001972050_00308_23408_0077_uint16_reduced_compressed.xml -r MERIS_uint16 --visible --traceback
python manage.py eoxs_dataset_register -d autotest_jenkins/data/meris/MER_FRS_1P_reduced/ENVISAT-MER_FRS_1PNPDE20060830_100949_000001972050_00423_23523_0079_uint16_reduced_compressed.tif -m autotest_jenkins/data/meris/MER_FRS_1P_reduced/ENVISAT-MER_FRS_1PNPDE20060830_100949_000001972050_00423_23523_0079_uint16_reduced_compressed.xml -r MERIS_uint16 --visible --traceback

python manage.py eoxs_collection_link --collection MER_FRS_1P_reduced --add MER_FRS_1PNPDE20060816_090929_000001972050_00222_23322_0058_uint16_reduced_compressed MER_FRS_1PNPDE20060822_092058_000001972050_00308_23408_0077_uint16_reduced_compressed MER_FRS_1PNPDE20060830_100949_000001972050_00423_23523_0079_uint16_reduced_compressed

python manage.py eoxs_dataset_register --collection MER_FRS_1P_reduced -d autotest_jenkins/data/meris/mosaic_MER_FRS_1P_reduced_RGB/mosaic_ENVISAT-MER_FRS_1PNPDE20060816_090929_000001972050_00222_23322_0058_RGB_reduced.tif -m autotest_jenkins/data/meris/mosaic_MER_FRS_1P_reduced_RGB/mosaic_ENVISAT-MER_FRS_1PNPDE20060816_090929_000001972050_00222_23322_0058_RGB_reduced.xml -r RGB --visible --traceback
python manage.py eoxs_dataset_register --collection MER_FRS_1P_reduced -d autotest_jenkins/data/meris/mosaic_MER_FRS_1P_reduced_RGB/mosaic_ENVISAT-MER_FRS_1PNPDE20060822_092058_000001972050_00308_23408_0077_RGB_reduced.tif -m autotest_jenkins/data/meris/mosaic_MER_FRS_1P_reduced_RGB/mosaic_ENVISAT-MER_FRS_1PNPDE20060822_092058_000001972050_00308_23408_0077_RGB_reduced.xml -r RGB --visible --traceback
python manage.py eoxs_dataset_register --collection MER_FRS_1P_reduced -d autotest_jenkins/data/meris/mosaic_MER_FRS_1P_reduced_RGB/mosaic_ENVISAT-MER_FRS_1PNPDE20060830_100949_000001972050_00423_23523_0079_RGB_reduced.tif -m autotest_jenkins/data/meris/mosaic_MER_FRS_1P_reduced_RGB/mosaic_ENVISAT-MER_FRS_1PNPDE20060830_100949_000001972050_00423_23523_0079_RGB_reduced.xml -r RGB --visible --traceback
#python manage.py eoxs_collection_link --series MER_FRS_1P_reduced --add mosaic_MER_FRS_1PNPDE20060816_090929_000001972050_00222_23322_0058_RGB_reduced mosaic_MER_FRS_1PNPDE20060822_092058_000001972050_00308_23408_0077_RGB_reduced mosaic_MER_FRS_1PNPDE20060830_100949_000001972050_00423_23523_0079_RGB_reduced --traceback

python manage.py eoxs_dataset_register -d autotest_jenkins/data/asar/ASA_WSM_1PNDPA20050331_075939_000000552036_00035_16121_0775.tiff -r ASAR --coverage-type=ReferenceableDataset --projection 4326 --extent 16.727605,-36.259107,22.301754,-31.984922 --visible --traceback
python manage.py eoxs_id_list --traceback

# Send some requests and compare results with expected results
python manage.py runserver 1>/dev/null 2>&1 &
sleep 3
PID=$!

curl -sS -o tmp "http://localhost:8000/ows?service=wcs&request=getcapabilities"
xmllint --format tmp > tmp1
curl -sS -o tmp "http://localhost:8000/ows?service=WCS&version=2.0.1&request=GetCapabilities"
xmllint --format tmp > tmp2
curl -sS -o tmp "http://localhost:8000/ows?service=WCS&version=2.0.1&request=DescribeCoverage&CoverageId=MER_FRS_1PNPDE20060822_092058_000001972050_00308_23408_0077_uint16_reduced_compressed"
xmllint --format tmp > tmp3
curl -sS -o tmp "http://localhost:8000/ows?service=WCS&version=2.0.1&request=DescribeEOCoverageSet&eoId=MER_FRS_1PNPDE20060822_092058_000001972050_00308_23408_0077_uint16_reduced_compressed"
xmllint --format tmp > tmp4

# Restart development server otherwise the GetCoverage request hangs forever
if ps --ppid $PID -o pid=; then
    kill `ps --ppid $PID -o pid=`
fi
python manage.py runserver 1>/dev/null 2>&1 &
sleep 3
PID=$!

curl -sS -o tmp "http://localhost:8000/ows?service=wcs&version=2.0.1&request=GetCoverage&CoverageId=MER_FRS_1PNPDE20060822_092058_000001972050_00308_23408_0077_uint16_reduced_compressed&format=image/tiff"

# Perform binary comparison only on reference platform
if [ $DB == "spatialite" ]; then
    diff tmp1 autotest_jenkins/expected/command_line_test_getcapabilities.xml
    diff tmp2 autotest_jenkins/expected/command_line_test_getcapabilities.xml
    diff tmp3 autotest_jenkins/expected/command_line_test_describecoverage.xml
    diff tmp4 autotest_jenkins/expected/command_line_test_describeeocoverageset.xml
fi
if [ $OS != "Ubuntu" ]; then
    diff tmp autotest_jenkins/expected/WCS20GetCoverageDatasetTestCase.tif
fi

rm tmp tmp1 tmp2 tmp3 tmp4
if ps --ppid $PID -o pid=; then
    kill `ps --ppid $PID -o pid=`
fi

python manage.py eoxs_collection_unlink --collection MER_FRS_1P_reduced --remove mosaic_MER_FRS_1PNPDE20060816_090929_000001972050_00222_23322_0058_RGB_reduced mosaic_MER_FRS_1PNPDE20060822_092058_000001972050_00308_23408_0077_RGB_reduced mosaic_MER_FRS_1PNPDE20060830_100949_000001972050_00423_23523_0079_RGB_reduced --traceback
python manage.py eoxs_dataset_deregister mosaic_MER_FRS_1PNPDE20060816_090929_000001972050_00222_23322_0058_RGB_reduced mosaic_MER_FRS_1PNPDE20060822_092058_000001972050_00308_23408_0077_RGB_reduced mosaic_MER_FRS_1PNPDE20060830_100949_000001972050_00423_23523_0079_RGB_reduced --traceback
python manage.py eoxs_id_list --traceback
python manage.py eoxs_id_check --traceback -t DatasetSeries MER_FRS_1P_reduced && echo "Test of 'eoxs_id_check' passed."
python manage.py eoxs_id_check --traceback ENVISAT-MER_FRS_1PNPDE20060816_090929_000001972050_00222_23322_0058_uint16_reduced_compressed && echo "Test of 'eoxs_id_check' passed."
python manage.py eoxs_id_check --traceback nonexistent_XYZ || echo "Test of 'eoxs_id_check' passed."
#python manage.py eoxs_id_check notused --traceback
#python manage.py eoxs_id_check ENVISAT-MER_FRS_1PNPDE20060816_090929_000001972050_00222_23322_0058_uint16_reduced_compressed --traceback
#python manage.py eoxs_id_check MER_FRS_1P_reduced --traceback
# TODO: re-activate once implemented
#python manage.py eoxs_collection_create -i test_sync -d autotest_jenkins/data/meris/MER_FRS_1P_reduced/ autotest_jenkins/data/meris/mosaic_MER_FRS_1P_reduced_RGB/ -p "*.tif" --traceback
#python manage.py eoxs_synchronize -a --traceback
python manage.py eoxs_id_list --traceback

# deregister all of MER_FRS_1P_reduced
python manage.py eoxs_dataset_deregister MER_FRS_1PNPDE20060816_090929_000001972050_00222_23322_0058_uint16_reduced_compressed MER_FRS_1PNPDE20060822_092058_000001972050_00308_23408_0077_uint16_reduced_compressed MER_FRS_1PNPDE20060830_100949_000001972050_00423_23523_0079_uint16_reduced_compressed

# test batch registration
echo "\"data\",\"metadata\",\"range_type_name\",\"visible\",\"collection\"
\"autotest_jenkins/data/meris/MER_FRS_1P_reduced/ENVISAT-MER_FRS_1PNPDE20060816_090929_000001972050_00222_23322_0058_uint16_reduced_compressed.tif\",\"autotest_jenkins/data/meris/MER_FRS_1P_reduced/ENVISAT-MER_FRS_1PNPDE20060816_090929_000001972050_00222_23322_0058_uint16_reduced_compressed.xml\",\"MERIS_uint16\",TRUE,\"MER_FRS_1P_reduced\"
\"autotest_jenkins/data/meris/MER_FRS_1P_reduced/ENVISAT-MER_FRS_1PNPDE20060822_092058_000001972050_00308_23408_0077_uint16_reduced_compressed.tif\",\"autotest_jenkins/data/meris/MER_FRS_1P_reduced/ENVISAT-MER_FRS_1PNPDE20060822_092058_000001972050_00308_23408_0077_uint16_reduced_compressed.xml\",\"MERIS_uint16\",TRUE,\"MER_FRS_1P_reduced\"
\"autotest_jenkins/data/meris/MER_FRS_1P_reduced/ENVISAT-MER_FRS_1PNPDE20060830_100949_000001972050_00423_23523_0079_uint16_reduced_compressed.tif\",\"autotest_jenkins/data/meris/MER_FRS_1P_reduced/ENVISAT-MER_FRS_1PNPDE20060830_100949_000001972050_00423_23523_0079_uint16_reduced_compressed.xml\",\"MERIS_uint16\",TRUE,\"MER_FRS_1P_reduced\"
" > batch.csv

python manage.py eoxs_dataset_register_batch batch.csv --traceback

rm batch.csv

# test --replace flag
python manage.py eoxs_dataset_register -d autotest_jenkins/data/meris/MER_FRS_1P_reduced/ENVISAT-MER_FRS_1PNPDE20060816_090929_000001972050_00222_23322_0058_uint16_reduced_compressed.tif -m autotest_jenkins/data/meris/MER_FRS_1P_reduced/ENVISAT-MER_FRS_1PNPDE20060816_090929_000001972050_00222_23322_0058_uint16_reduced_compressed.xml -r MERIS_uint16 --visible --replace --traceback
python manage.py eoxs_dataset_register -d autotest_jenkins/data/meris/MER_FRS_1P_reduced/ENVISAT-MER_FRS_1PNPDE20060822_092058_000001972050_00308_23408_0077_uint16_reduced_compressed.tif -m autotest_jenkins/data/meris/MER_FRS_1P_reduced/ENVISAT-MER_FRS_1PNPDE20060822_092058_000001972050_00308_23408_0077_uint16_reduced_compressed.xml -r MERIS_uint16 --visible --replace --traceback
python manage.py eoxs_dataset_register -d autotest_jenkins/data/meris/MER_FRS_1P_reduced/ENVISAT-MER_FRS_1PNPDE20060830_100949_000001972050_00423_23523_0079_uint16_reduced_compressed.tif -m autotest_jenkins/data/meris/MER_FRS_1P_reduced/ENVISAT-MER_FRS_1PNPDE20060830_100949_000001972050_00423_23523_0079_uint16_reduced_compressed.xml -r MERIS_uint16 --visible --replace --traceback

# tests using cci_soilmoisture data
# define a range type for real SM data
python manage.py eoxs_rangetype_load -i autotest_jenkins/data/cci_soilmoisture/sm_rangetype.json
# create empty collections
python manage.py eoxs_collection_create -i SoilMoisture
python manage.py eoxs_collection_create -i SoilMoistureRGB
# add datasources to the collections: a template to look up files
python manage.py eoxs_collection_datasource -i SoilMoisture -s "`pwd`/autotest_jenkins/data/cci_soilmoisture/*tif" -t "`pwd`/autotest_jenkins/data/cci_soilmoisture/{basename}.xml" -t "`pwd`/autotest_jenkins/data/cci_soilmoisture/sm_rangetype.conf"
python manage.py eoxs_collection_datasource -i SoilMoistureRGB -s "`pwd`/autotest_jenkins/data/cci_soilmoisture/*png" -t "`pwd`/autotest_jenkins/data/cci_soilmoisture/{basename}.xml" -t "`pwd`/autotest_jenkins/data/cci_soilmoisture/rgb_rangetype.conf"
# synchronize the collections: register/unregister depending on the files in the datasources
python manage.py eoxs_collection_synchronize -i SoilMoisture
python manage.py eoxs_collection_synchronize -i SoilMoistureRGB

# Run Selenium
echo "**> running Selenium tests ..."
#TODO
